@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # coding: utf-8
 # /*##########################################################################
 #
@@ -32,13 +32,12 @@ Test coverage dependencies: coverage, lxml.
 """
 
 __authors__ = ["Jérôme Kieffer", "Thomas Vincent"]
-__date__ = "28/07/2017"
+__date__ = "03/03/2023"
 __license__ = "MIT"
 
-import distutils.util
 import logging
 import os
-import subprocess
+from argparse import ArgumentParser
 import sys
 import time
 import unittest
@@ -75,13 +74,18 @@ def createBasicHandler():
 
 # Use an handler compatible with unittests, else use_buffer is not working
 logging.root.addHandler(createBasicHandler())
+
+# Capture all default warnings
 logging.captureWarnings(True)
+import warnings
+warnings.simplefilter('default')
 
 logger = logging.getLogger("run_tests")
 logger.setLevel(logging.WARNING)
 
 logger.info("Python %s %s", sys.version, tuple.__itemsize__ * 8)
-
+if sys.version_info.major < 3:
+    logger.error("FabIO no more support Python2")
 
 try:
     import resource
@@ -93,6 +97,7 @@ try:
     import importlib
     importer = importlib.import_module
 except ImportError:
+
     def importer(name):
         module = __import__(name)
         # returns the leaf module, instead of the root module
@@ -102,7 +107,6 @@ except ImportError:
             module = getattr(module, subname)
             return module
 
-
 try:
     import numpy
 except Exception as error:
@@ -110,7 +114,6 @@ except Exception as error:
 else:
     logger.info("Numpy %s", numpy.version.version)
 
-
 try:
     import h5py
 except Exception as error:
@@ -118,22 +121,9 @@ except Exception as error:
 else:
     logger.info("h5py %s", h5py.version.version)
 
-
-def get_project_name(root_dir):
-    """Retrieve project name by running python setup.py --name in root_dir.
-
-    :param str root_dir: Directory where to run the command.
-    :return: The name of the project stored in root_dir
-    """
-    logger.debug("Getting project name in %s", root_dir)
-    p = subprocess.Popen([sys.executable, "setup.py", "--name"],
-                         shell=False, cwd=root_dir, stdout=subprocess.PIPE)
-    name, _stderr_data = p.communicate()
-    logger.debug("subprocess ended with rc= %s", p.returncode)
-    return name.split()[-1].decode('ascii')
-
-
 PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))
+sys.path.insert(0, PROJECT_DIR)
+from bootstrap import get_project_name, build_project
 PROJECT_NAME = get_project_name(PROJECT_DIR)
 logger.info("Project name: %s", PROJECT_NAME)
 
@@ -193,7 +183,7 @@ class ProfileTextTestResult(unittest.TextTestRunner.resultclass):
         else:
             memusage = 0
         self.logger.info("Time: %.3fs \t RAM: %.3f Mb\t%s",
-            time.time() - self.__time_start, memusage, test.id())
+                         time.time() - self.__time_start, memusage, test.id())
 
 
 def report_rst(cov, package, version="0.0.0", base=""):
@@ -210,9 +200,13 @@ def report_rst(cov, package, version="0.0.0", base=""):
     os.close(fd)
     cov.xml_report(outfile=fn)
 
-    from lxml import etree
+    try:
+        from lxml import etree
+    except ImportError:
+        import xml.etree.ElementTree as etree
+
     xml = etree.parse(fn)
-    classes = xml.xpath("//class")
+    classes = xml.findall(".//class")
 
     line0 = "Test coverage report for %s" % package
     res = [line0, "=" * len(line0), ""]
@@ -231,7 +225,7 @@ def report_rst(cov, package, version="0.0.0", base=""):
         fname = cl.get("filename")
         if not os.path.abspath(fname).startswith(base):
             continue
-        lines = cl.find("lines").getchildren()
+        lines = list(cl.find("lines"))
         hits = [int(i.get("hits")) for i in lines]
 
         sum_hits = sum(hits)
@@ -268,37 +262,6 @@ def is_debug_python():
     return hasattr(sys, "gettotalrefcount")
 
 
-def build_project(name, root_dir):
-    """Run python setup.py build for the project.
-
-    Build directory can be modified by environment variables.
-
-    :param str name: Name of the project.
-    :param str root_dir: Root directory of the project
-    :return: The path to the directory were build was performed
-    """
-    platform = distutils.util.get_platform()
-    architecture = "lib.%s-%i.%i" % (platform,
-                                     sys.version_info[0], sys.version_info[1])
-    if is_debug_python():
-        architecture += "-pydebug"
-
-    if os.environ.get("PYBUILD_NAME") == name:
-        # we are in the debian packaging way
-        home = os.environ.get("PYTHONPATH", "").split(os.pathsep)[-1]
-    elif os.environ.get("BUILDPYTHONPATH"):
-        home = os.path.abspath(os.environ.get("BUILDPYTHONPATH", ""))
-    else:
-        home = os.path.join(root_dir, "build", architecture)
-
-    logger.warning("Building %s to %s", name, home)
-    p = subprocess.Popen([sys.executable, "setup.py", "build"],
-                         shell=False, cwd=root_dir)
-    logger.debug("subprocess ended with rc= %s", p.wait())
-    return home
-
-
-from argparse import ArgumentParser
 parser = ArgumentParser(description='Run the tests.')
 
 parser.add_argument("--installed",
@@ -325,7 +288,6 @@ parser.add_argument("test_name", nargs='*',
 options = parser.parse_args()
 sys.argv = [sys.argv[0]]
 
-
 test_verbosity = 1
 use_buffer = True
 if options.verbose == 1:
@@ -357,15 +319,16 @@ if (os.path.dirname(os.path.abspath(__file__)) ==
     removed_from_sys_path = sys.path.pop(0)
     logger.info("Patched sys.path, removed: '%s'", removed_from_sys_path)
 
-
-# import module
 if options.installed:  # Use installed version
+    for bad_path in (".", os.getcwd(), os.path.abspath(".")):
+        if bad_path in sys.path:
+            sys.path.remove(bad_path)
     try:
         module = importer(PROJECT_NAME)
-    except:
-        raise ImportError(
-            "%s not installed: Cannot run tests on installed version" %
-            PROJECT_NAME)
+    except Exception:
+        logger.error("Cannot run tests on installed version: %s not installed or raising error.",
+                     PROJECT_NAME)
+        raise
 else:  # Use built source
     build_dir = build_project(PROJECT_NAME, PROJECT_DIR)
 
@@ -373,11 +336,9 @@ else:  # Use built source
     logger.warning("Patched sys.path, added: '%s'", build_dir)
     module = importer(PROJECT_NAME)
 
-
 PROJECT_VERSION = getattr(module, 'version', '')
 PROJECT_PATH = module.__path__[0]
 
-
 # Run the tests
 runnerArgs = {}
 runnerArgs["verbosity"] = test_verbosity
@@ -394,11 +355,6 @@ logger.warning("Test %s %s from %s",
 test_module_name = PROJECT_NAME + '.test'
 logger.info('Import %s', test_module_name)
 test_module = importer(test_module_name)
-utilstest = importer(test_module_name + ".utilstest")
-UtilsTest = getattr(utilstest, "UtilsTest")
-UtilsTest.image_home = os.path.join(PROJECT_DIR, 'testimages')
-UtilsTest.testimages = os.path.join(PROJECT_DIR, "all_testimages.json")
-UtilsTest.script_dir = os.path.join(PROJECT_DIR, "scripts")
 
 test_suite = unittest.TestSuite()
 
@@ -421,7 +377,6 @@ if result.wasSuccessful():
 else:
     exit_status = 1
 
-
 if options.coverage:
     cov.stop()
     cov.save()
